using System;
using System.Collections.Generic;
using System.Text;

class cCallEvent
{
    public const int IN_BOUND_CALL_REJECT = 0;
    public const int IN_BOUND_CALL_CONNECT_USER = 1;
    public const int IN_BOUND_CALL_PUT_IN_QUEUE = 2;

    public void OnIncomingCallFromLine(ref cCallData objCallData, ref cLineData objLineData, string sDialNo)
    {
	    if(objLineData.m_nCallType == IN_BOUND_CALL_REJECT)
	    {
		    cServerSIP.m_objServerSIP.CloseCall(ref objCallData);
		    return;
	    }

        if (objLineData.m_nCallType == IN_BOUND_CALL_CONNECT_USER)
	    {
		    cServerSIP.m_objServerSIP.DialCallToUser(ref objCallData, objLineData.m_sCallTo);
		    return;
	    }

        if (objLineData.m_nCallType == IN_BOUND_CALL_PUT_IN_QUEUE)
	    {
		    cQueueData objQueueData = cServerSIP.m_objServerSIP.m_objQueueList.FindQueue(objLineData.m_sCallTo);

		    if(objQueueData == null)
		    {
			    cServerSIP.m_objServerSIP.CloseCall(ref objCallData);
		    }
		    else
		    {
                objCallData.m_nCallType = cServerSIP.CALL_TYPE_QUEUE;
			    OnIncomingCallQueueType(ref objCallData, objQueueData.m_unQueueId);
		    }
	    }
    }

    public void OnIncomingCallFromUser(ref cCallData objUserCallData, cUserData objUserData, string sDialNo) 
    {
	    if(objUserData.m_bStealthListener)
	    {
            objUserCallData.m_nCallType = cServerSIP.CALL_TYPE_STEALTH;
		    OnIncomingCallStealthType(ref objUserCallData, ref objUserData, sDialNo);
	    }
	    else
	    {
            objUserCallData.m_nCallType = cServerSIP.CALL_TYPE_REGULAR;
		    OnIncomingCallRegularType(ref objUserCallData, ref objUserData, sDialNo);	
	    }
    }

    //////////////////////////////////////////

        public void OnIncomingCallQueueType(ref cCallData objCallData, uint unQueueId)
    {
        objCallData.m_unQueueId = unQueueId;
        cServerSIP.m_objServerSIP.AcceptCall(ref objCallData);
    }

    public void OnIncomingCallStealthType(ref cCallData objCallData, ref cUserData objUserData, string sDialNo)
    {
	    objCallData.m_sStealthDialNo = sDialNo;
	    objCallData.m_sStealthDialNo.ToLower();

        if (cServerSIP.m_objServerSIP.AcceptCall(ref objCallData))
            objUserData.m_nCallCount++;
    }

    public void OnIncomingCallRegularType(ref cCallData objCallData, ref cUserData objUserData, string sDialNo)
    {
	    int nDialPlanIndex = cServerSIP.m_objServerSIP.m_objDialPlanList.SearchDialPlan(sDialNo);
    	
	    if(nDialPlanIndex != -1)
	    {
		    string sLineName = "";
		    sDialNo = cServerSIP.m_objServerSIP.m_objDialPlanList.ApplyDialPlan(nDialPlanIndex, sDialNo, ref sLineName);

            if (cServerSIP.m_objServerSIP.DialCallToLine(ref objCallData, sDialNo, sLineName) != null)
                objUserData.m_nCallCount++;
	    }
	    else
	    {
            if (cServerSIP.m_objServerSIP.DialCallToUser(ref objCallData, sDialNo) != null)
                objUserData.m_nCallCount++;
	    }		
    }

    ////////////////////////////////////////////////////

    public void OnAcceptIncomingCallConnected(ref cCallData objCallData) 
    {
	    objCallData.m_bConnected = true;
    	
	    switch(objCallData.m_nCallType)
	    {
            case cServerSIP.CALL_TYPE_STEALTH:
			    OnConnectedCallStealthType(ref objCallData);
			    break;

            case cServerSIP.CALL_TYPE_QUEUE:
			    OnConnectedCallQueueType(ref objCallData);
			    break;

            case cServerSIP.CALL_TYPE_REGULAR:
			    break;
	    }
    }

    public void OnDialCallConnected(ref cCallData objCallData) 
    {
	    objCallData.m_bConnected = true;
    							
	    switch(objCallData.m_nCallType)
	    {
            case cServerSIP.CALL_TYPE_REGULAR:
			    OnConnectedCallRegularType(ref objCallData);
			    break;

            case cServerSIP.CALL_TYPE_QUEUE:
			    break;

            case cServerSIP.CALL_TYPE_STEALTH:
			    break;
	    }
    }

    ////////////////////////////////////////////////////////

    public void OnConnectedCallRegularType(ref cCallData objCallData)
    {
	    cCallData objBindCallData = objCallData.m_objBindCallData;

        if (objBindCallData != null && objBindCallData.m_bConnected == false)
        {
            if (cServerSIP.m_objServerSIP.AcceptCall(ref objBindCallData) == false)
            {
                cServerSIP.m_objServerSIP.CloseCall(ref objBindCallData);
                cServerSIP.m_objServerSIP.CloseCall(ref objCallData);

                return;
            }
        }
    	
	    cServerSIP.m_objServerSIP.StartVoiceStream(ref objCallData, ref objBindCallData);
    	
	    cServerSIP.m_objServerSIP.m_objStealthList.TryCaptureCall(ref objCallData);
	    cServerSIP.m_objServerSIP.m_objStealthList.TryCaptureCall(ref objBindCallData);
    }

    public void OnConnectedCallStealthType(ref cCallData objCallData)
    {
	    cServerSIP.m_objServerSIP.m_objStealthList.AddCallToStealthList(ref objCallData);

	    int nStealthSessionId = cServerSIP.m_objServerSIP.m_objServerData.GetStealthSessionId();
        cServerSIP.m_objServerSIP.AddCallToSession(ref objCallData, nStealthSessionId);
    }

    public void OnConnectedCallQueueType(ref cCallData objCallData)
    {
	    cServerSIP.m_objServerSIP.m_objQueueList.AddCallToQueue(ref objCallData);
    }

    ////////////////////////////////////////////////////////

    public void OnDialCallTimeOut(ref cCallData objCallData) 
    {
	    cCallData objBindCallData = objCallData.m_objBindCallData;

	    if(objBindCallData != null)
	    {
		    cServerSIP.m_objServerSIP.SendResponse(ref objBindCallData, 408, "Request Timeout");
		    cServerSIP.m_objServerSIP.CloseCall(ref objBindCallData);
	    }

	    cServerSIP.m_objServerSIP.CloseCall(ref objCallData);
    }

    public void OnAcceptCallTimeOut(ref cCallData objCallData)
    {
	    OnDisconnectCall(ref objCallData);
    }

    public void OnDisconnectCall(ref cCallData objCallData) 
    {
	    cCallData objBindCallData = objCallData.m_objBindCallData;
    	
	    if(objBindCallData != null)
		    cServerSIP.m_objServerSIP.CloseCall(ref objBindCallData);
    	
	    cServerSIP.m_objServerSIP.CloseCall(ref objCallData);
    }

    public void OnIncomingCallCancel(ref cCallData objCallData) 
    {
	    OnDisconnectCall(ref objCallData);
    }

    public void OnSessionLost(ref cCallData objCallData) 
    {
	    OnDisconnectCall(ref objCallData);
    }
}